%   *******************************
%   * MAIN METHOD FOR NATURAL GAS *
%   * PIPELINE CALCULATION        *  
%   *******************************
%   
clear all
clc

%-----------------------------------------------------
%   DETERMINE CASE
%-----------------------------------------------------
inletbc.case=input('steady-state[1],oscillating flow[2],closing valve[3] or opening valve[4]: ');

%***PRE-PROCESSING***
%-----------------------------------------------------
%   GLOBAL CONSTANTS
%   R-Universal gas constant
%   g-gravity
%-----------------------------------------------------  
g=9.81;
R=8314; 
%-----------------------------------------------------
%   READING DATA
%-----------------------------------------------------  
% reading data for pipeline
profile=xlsread('profile_europipe_2.xls');

% reading data for the different flows
if inletbc.case==1
    inletmassflow=70.6;
elseif inletbc.case==2
    testcase1=xlsread('test1_04feb.xls');
    inletmassflow=testcase1(:,1);
elseif inletbc.case==3
    inletmassflow=xlsread('shutdown_inletmass.xls');    
elseif inletbc.case==4
    inletmassflow=xlsread('startup_case.xls');
else
    disp('a case must be given')
    break
end


%-----------------------------------------------------
%   PIPELINE PROPERTIES
%-----------------------------------------------------

%   pipeline.length-        length of pipeline from K?rst?-Bokn[m]
%   pipeline.nodes-         number of nodes [#]
%   pipeline.diameter-      diameter of the pipe[m]
%   pipeline.area-          area of the pipe[m2]
%   pipeline.height-        height of the pipeline at each initial node[m]
%   pipeline.initial_nodes- number of nodes from the original pipeline
%                           profile
%   pipeline.dx-            length between the nodes;[m]
%   pipeline.alfa-          agle of the pipeline piece[degrees]
%   nodes_pr_dx-            number of nodes for each pipeline segment from our
%                           pipeline profile input.
%   pipeline.roughness-     Pipeline roughness
%-----------------------------------------------------

%---pipeline constants---------------------------------
pipeline.diameter=1.016;
pipeline.area=0.25*pi*pipeline.diameter^2;
pipeline.roughness=5e-6;

% pipeline properties
pipeline.height=profile(:,3);
pipeline.length=profile(length(profile(:,1)),2);            
pipeline.initial_nodes=profile(length(profile(:,1)),1);
total_number_of_nodes= input('the total number of wanted nodes in calculation: ');
nodes_pr_dx=round(total_number_of_nodes/pipeline.initial_nodes);
pipeline.dx_initial=pipeline.length/pipeline.initial_nodes;
pipeline.dx=pipeline.dx_initial/nodes_pr_dx;
pipeline.nodes=nodes_pr_dx*(pipeline.initial_nodes-1)+1;
disp(sprintf('total number of pipeline nodes: %3.4g', pipeline.nodes));

for i=1:(profile(length(profile(:,1)),1)-1)
    pipeline.alfa(i)=atan((pipeline.height(i+1)-pipeline.height(i))/pipeline.dx_initial);
end

%-----------------------------------------------------
%   NATURAL GAS PROPERTIES
%-----------------------------------------------------

%   gas.p- gas pressure[Pa]                           
%   gas.Tc- critical temperature gas[K]
%   gas.rho-gas density[kg/m3]
%   gas.T- gas temperature[K]
%   R- universal gas constant[J/kmol*K]
%   gas.M- molecular mass gas[g/mol]
%   gas.R- gas constant
%   gas.Z- compressibility factor
%   gas.wavespeed- acustic wavespeed
%-----------------------------------------------------
if inletbc.case==1
    gas.p_init=180.8623657*1e5;
elseif inletbc.case==2
    gas.p_init=180.8623657*1e5;
elseif inletbc.case==3
    gas.p_init=185.431121826172*1e5;
elseif inletbc.case==4
    gas.p_init=147.213424682617*1e5;
end
gas.Tc=191;
gas.pc=46.4e5;
gas.T_init=273.15+33;
gas.Tr=gas.T_init/gas.Tc;
gas.pr=gas.p_init/gas.pc;
gas.MW=18.01;
gas.R=R/gas.MW;
gas.Z=compressibilityfactor(gas.pr,gas.Tr);
gas.wavespeed=sqrt(gas.Z*gas.T_init*gas.R);
gas.rho=gas.p_init/(gas.Z*gas.R*gas.T_init);
gas.my=viscosity(gas);
disp(sprintf('the acoustic wavespeed at ICs is set to: %3.4g', gas.wavespeed));
disp(sprintf('the initial gas density is: %3.4g', gas.rho));
%-----------------------------------------------------
%   INITIAL CONDITIONS/STEADY-STATE
%   MSM3_D- Million standard cubic metres per day
%   steady_massflow- the flowrate in the 
%-----------------------------------------------------

%***steady state calculations*******
MSM3_D=inletmassflow(1);
massflow_corr=8.831;
steady_massflow=MSM3_D*massflow_corr;
disp(sprintf('the initial gas velocity: %3.4g', (steady_massflow/(gas.rho*pipeline.area))));
pipeline.s=2*g*min(pipeline.dx)*sin(pipeline.alfa)./gas.wavespeed^2;

initial.pressure(1)=gas.p_init;
friction=frictionfactor(steady_massflow,gas,pipeline);
C=friction*gas.wavespeed^2*steady_massflow^2*pipeline.dx/(pipeline.diameter*pipeline.area^2);

%*****case 1*****
if nodes_pr_dx==1
disp('only one node each measuring point')
    for node=2:pipeline.nodes
            s=pipeline.s(node-1);
        if s==0
            initial.pressure(node)=sqrt(initial.pressure(node-1)^2-C);
        else
            initial.pressure(node)=sqrt((initial.pressure(node-1)^2-....
                C*((exp(s)-1)./s))/exp(s));
        end
    end
else
    for node=2:pipeline.nodes
          for n=0:pipeline.initial_nodes-2
                if node>=(2+nodes_pr_dx*n) && node<=(nodes_pr_dx*(1+n))
                    s=pipeline.s(n+1);
                end
          end
        if s==0
            initial.pressure(node)=sqrt(initial.pressure(node-1)^2-C);
        else
            initial.pressure(node)=sqrt((initial.pressure(node-1)^2-....
                C*((exp(s)-1)./s))/exp(s));
        end
    end
end

%__________INITIAL PLOTS______________
    figure(1)
    subplot(2,1,2)
    hold on
    
    subplot(2,1,1)
    title('PIPE PROFILE')
    plot(linspace(0,pipeline.length,length(pipeline.height)),pipeline.height);
    %clearaxis([0 12500 -100 150])
    xlabel('pipeline length[m]')
    ylabel('heightprofile[m]')    
 
    subplot(2,1,2)
    title('PLOT OF INITIAL PRESSUREDISTRIBUTION')
    plot(linspace(0,pipeline.length,pipeline.nodes),(initial.pressure)/10^5)

    xlabel('meter[m]')
    ylabel('Pressure[Bar]')
    
    hold off
%______________________________________

%-----------------------------------------------------
% TIME EVALUATION
%-----------------------------------------------------
dt=pipeline.dx/gas.wavespeed;

%-----min simulation time---------------------------
t_min=pipeline.length/gas.wavespeed;

%-----time evaluation in seconds----------------------
 seconds=input('insert the number of secounds you want to do iterate over: ');
timesteps=ceil(seconds*1/dt);



%-----------------------------------------------------
%   BOUNDARY CONDITION
%-----------------------------------------------------


    %  *********************
    %  *VALVE              *
    %  *opening and closing*
    %  *********************

if inletbc.case==1
    inletbc.massflow=steady_massflow;
    disp('ateady state')

elseif inletbc.case==2
        inletbc.osc_case=input('case 1/2 or case 3/4 ?[1/2]: ');
        if inletbc.osc_case==1
            inletbc.massflow=inletmassflow;
        elseif inletbc.osc_case==2
            period=input('insert oscillation period[min]: ');
            inletbc.oscperiod=1/(period*60)*2*pi;
            inletbc.oscmagnitude=input('insert the magnitude of the oscillations[kg/s]: ');
            inletbc.massflow=steady_massflow;
        end
elseif inletbc.case==3
    inletbc.close_case=input('case6/7 or case 8/9 [1/2]: ');
    if inletbc.close_case==1
        inletbc.closingtime=input('insert valve closingtime in sec: ');
        inletbc.massflow(1)=steady_massflow;
    elseif inletbc.close_case==2
        inletbc.massflow=inletmassflow;  
    end


elseif inletbc.case==4
    inletbc.massflow=inletmassflow;
end

outletbc.pressure=initial.pressure(pipeline.nodes);
outletbc.massflow=0;
mass_init=1;
if sum(inletbc.massflow)~=0
    inletbc.type='massflow is given at inlet';
else 
    inletbc.type='pressure is given at inlet';
end

if outletbc.massflow ~=0
    outletbc.type='massflow is given at outlet';
else
    outletbc.type='pressure is given at outlet';
end

%-----------------------------------------------------


%***MAIN PROCESSING***

fluid.massflow=ones(1,pipeline.nodes)*steady_massflow;
fluid.pressure=initial.pressure;

%-----------------------------------------------------
%   EVALUATION OF THE INERTIAL MULTIPLIER
%-----------------------------------------------------
inertia_m=input('using inertia multiplier? [Y/N]: ','s');


%inertia_m='Y';


if inertia_m=='Y'
    %fi=input('insert pressure acceptance error, fi: ');
    fi=0.05;
    alfa=inertial_multiplier(fi,inletbc,fluid,pipeline,gas,massflow_corr);
    alfa=ceil(alfa)-1;
    %alfa=round(alfa);
else
    alfa=1;
end

%-----------------------------------------------------

%testcase1_outletpressure(1)=fluid.pressure(1);

    node_save=input('Save result for each node number: ');
    sec_save=input('save for each second number: ');
    time_save=round(sec_save/dt);
    row=round(pipeline.nodes/node_save)+1;
    column=ceil(timesteps/time_save);
    massflow_save=zeros(column,row);
    pressure_save=zeros(column,row);



     fig2=figure(2);
     title('dynamic plot of pressure and massflow')
     hold on
%         numframes=timesteps;
%         winsize=get(fig2,'position');
%         winsize(1:2)=[0 0];
%         Movie=moviein(numframes,fig2,winsize);
%         set(fig2,'NextPlot','replacechildren')
 

it=2;
if inertia_m~='Y'
    alfa=1;
    tic
    contd=0;
    for time=1:timesteps

        sec=time*dt;
        if mod(time,100)==0
            disp(sprintf('time in seconds: %3.4g', round(sec)));
        end

          fluid=solver(fluid,pipeline,gas,inletbc,outletbc,time,nodes_pr_dx,dt,alfa,massflow_corr);

          fig2;
          subplot(2,1,1)
          p=plot(linspace(0,pipeline.length,pipeline.nodes),fluid.massflow);
          ylabel('massflow[kg/s]')
          xlabel('meter[m]')

          subplot(2,1,2)
          l=plot(linspace(0,pipeline.length,pipeline.nodes),fluid.pressure/10^5);
          xlabel('meter[m]')
          ylabel('pressure[bar]')

        %--------------------
        %   movie object
        % Movie(:,time)=getframe(fig2);
        % aviobject=addframe(aviobject,Frames);
        %---------------------------------
        
        if time==100
            time_eval=toc;
            disp(sprintf('elapsed time first 100 timesteps: %3.4g',time_eval));
            disp(sprintf('approximately total time for calculation: %3.4g',time_eval*timesteps/(100*3600)));
            while contd==0
                answer=input('do you wish to continue?[Y/N]: ','s');
                if answer=='Y' || answer=='y'
                    contd=1;
                elseif answer=='N' || answer=='n'
                    contd=2;
                    break
                else
                    answer=input('do you wish to continue?[Y/N]: ','s');
                end
            end
        if contd==2
            break
        end
        end

            if time==1
                 for iter=node_save:node_save:pipeline.nodes

                    massflow_save(1,1)=fluid.massflow(1);
                    massflow_save(1,iter/node_save+1)=fluid.massflow(iter);
                    massflow_save(1,row)=fluid.massflow(pipeline.nodes);
                    pressure_save(1,1)=fluid.pressure(1);
                    pressure_save(1,iter/node_save+1)=fluid.pressure(iter);
                    pressure_save(1,row)=fluid.pressure(pipeline.nodes);
                 end


            elseif mod(time,time_save)==0

                for iter=node_save:node_save:pipeline.nodes
                    massflow_save(it,1)=fluid.massflow(1);
                    massflow_save(it,iter/node_save+1)=fluid.massflow(iter);
                    massflow_save(it,row)=fluid.massflow(pipeline.nodes);
                    pressure_save(it,1)=fluid.pressure(1);
                    pressure_save(it,iter/node_save+1)=fluid.pressure(iter);
                    pressure_save(it,row)=fluid.pressure(pipeline.nodes);

                end

                it=it+1;
            end
         pause(0.01);
        if time~=timesteps
              delete(p)
              delete(l)
         end 
    end

elseif inertia_m=='Y'
    iteration=1;
    time=alfa*dt;
    while time<seconds
        fluid=solver(fluid,pipeline,gas,inletbc,outletbc,time,nodes_pr_dx,dt,alfa,massflow_corr);
       
        if mod(iteration,100)==0
            disp(sprintf('time in seconds: %3.4g', round(time*dt)));
            alfa=inertial_multiplier(fi,inletbc,fluid,pipeline,gas,massflow_corr);
        end

       %------plotting-----------------------------------
        fig2;
       subplot(2,1,1)
       p=plot(linspace(0,pipeline.length,pipeline.nodes),fluid.massflow);
       ylabel('massflow[kg/s]')
       xlabel('meter[m]')

       subplot(2,1,2)
       l=plot(linspace(0,pipeline.length,pipeline.nodes),fluid.pressure/10^5);
       xlabel('meter[m]')
       ylabel('pressure[bar]')
       %------------------------------
       
            if iteration==1
                 for iter=node_save:node_save:pipeline.nodes
                    massflow_save(1,1)=fluid.massflow(1);
                    massflow_save(1,iter/node_save+1)=fluid.massflow(iter);
                    massflow_save(1,row)=fluid.massflow(pipeline.nodes);
                    pressure_save(1,1)=fluid.pressure(1);
                    pressure_save(1,iter/node_save+1)=fluid.pressure(iter);
                    pressure_save(1,row)=fluid.pressure(pipeline.nodes);
                 end
            elseif mod(iteration,time_save)==0
                for iter=node_save:node_save:pipeline.nodes
                    massflow_save(it,1)=fluid.massflow(1);
                    massflow_save(it,iter/node_save+1)=fluid.massflow(iter);
                    massflow_save(it,row)=fluid.massflow(pipeline.nodes);
                    pressure_save(it,1)=fluid.pressure(1);
                    pressure_save(it,iter/node_save+1)=fluid.pressure(iter);
                    pressure_save(it,row)=fluid.pressure(pipeline.nodes);
                end
            end
            pause(0.01);
            %delete(p)
            %delete(l)
              time=time+alfa*dt;
            iteration=iteration+1;

    end
end
    
    
    
    
%***POST PROCESSING***
%     massflowsaving=input('insert name of saved massflow file: ','s');
     %save testcase6_massflow_new.dat massflow_save -ascii
%     pressuresaving=input('insert name of saved pressure file: ','s')
     %save testcase6_pressure_new.dat pressure_save -ascii
% 
% 
% figure(4)
% title('INLET-OUTLET PRESSURE GIVEN VALUES')
% hold on
% xlabel('Time')
% ylabel('Pressure[bar]')
% starttime=datenum('02:56:08');
% stoptime=datenum('03:56:08');
% xData = linspace(starttime,stoptime,61);
% set(gca,'XTick',xData)
% plot(xData,inletpressure,'-*',xData,outletpressure,'o')
% datetick('x','HH:MM:SS','keeplimits')
% legend('Inletpressure','outletpressure')
% 
% figure(8)
% hold on
% title('INLETPRESSURE AND MASSFLOW')
% starttime=datenum('02:56:08');
% stoptime=datenum('03:56:08');
% xData = linspace(starttime,stoptime,length(inletpressure));
% hl1=line(xData,inletpressure,'Color','g');
% ylabel('Pressure')
% ax1=gca;
% set(gca,'XTick',xData)
% datetick('x','HH:MM:SS','keeplimits')
% set(ax1,'XColor','k','YColor','k')
% ax2=axes('Position',get(ax1,'Position'),'XAxislocation','bottom','YAxislocation','right','Color','none','XColor','k','YColor','b');
% hl2=line(xData,massflow_corr*inletmassflow,'Color','b','Parent',ax2);
% ylabel('Massflow')
% set(gca,'XTick',xData);
% datetick('x','HH:MM:SS','keeplimits');


%reading time data from dat file(from excel sheet)
% fid=fopen('timedata.dat','rt');
% C=textscan(fid,'%s %s %s %s %s %s','delimiter','.','CollectOutput',1);
% fclose(fid)

% figure(4)
% title('inletpressure and massflow with 5000 nodes 90 seconds')
% ylabel('pressure[Bar]')
% hold on
% hl1=line(xData,inletpressure_5000/1e5,'Color','k');
% set(gca,'XTick',xData)
% datetick('x','HH:MM:SS','keeplimits')
% ax1=gca;
% ax2=axes('Position',get(ax1,'Position'),'XAxislocation','bottom','YAxislocation','right','Color','none','XColor','k','YColor','r');
% hl2=line(xData,inletmassflow_5000,'Color','r','Parent',ax2);
% set(gca,'XTick',xData)
% datetick('x','HH:MM:SS','keeplimits')

%close(fig2);
%aviobject = close(aviobject);
%hold off

%movie(fig2,Movie,1,3)
%save testmovie.avi Movie
%movie2avi(Movie, 'test.avi', 'compression', 'None');

%mpgwrite(Frames,jet,'test1.mpg')
%mpgwrite(Frames,'testmovie.mpg');
%figure(3)
%plot(firstval_massflow)
%plot(pipeline.lengthprofile,fluid.massflow)m
%disp('  Massflow')
%disp(fluid.massflow)
%disp('  Pressure')
%disp(fluid.pressure)
% color=hsv(6);
% load testcase9_massflow.dat
% figure(3)
% hold on
% title('Massflow Case 9')
% ylabel('Massflow[kg/s]')
% xlabel('length')
% axis([0 12200 -400 700])
% it=2;
% plot(linspace(1,12200,length(testcase9_massflow(1,:))),testcase9_massflow(4,:),'color',color(1,:))
% for i=16:15:76
% plot(linspace(1,12200,length(testcase9_massflow(1,:))),testcase9_massflow(i+1,:),'Color',color(it,:))
% it=it+1;
% end
% % plot(linspace(1,12200,length(testcase8_massflow(1,:))),testcase8_massflow(16,:),'color',color(5,:))
% % plot(linspace(1,12200,length(testcase8_massflow(1,:))),testcase8_massflow(30,:),'color',color(6,:))
% legend('1 minute','5 minutes','10 minutes','15 minutes','20 minutes','25 minutes')
% 
% load testcase8_pressure.dat
% figure(4)
% hold on
% 
% title('Pressure Case 8')
% ylabel('Pressure[Bar]')
% xlabel('length')
% axis([0 12200 180 190])
% it=2;
% plot(linspace(1,12200,length(testcase8_pressure(1,:))),testcase8_pressure(7,:)./1e5,'Color',color(1,:))
% for i=16:15:76
% plot(linspace(1,12200,length(testcase8_pressure(1,:))),testcase8_pressure(i,:)./1e5,'Color',color(it,:))
% it=it+1;
% end
% % plot(linspace(1,12200,length(testcase8_pressure(1,:))),testcase8_pressure(16,:)./1e5,'Color',color(5,:))
% % plot(linspace(1,12200,length(testcase8_pressure(1,:))),testcase8_pressure(30,:)./1e5,'Color',color(6,:))
% legend('1 minute','5 minutes','10 minutes','15 minutes','20 minutes','25 minutes')
% 
% 
% figure(6)
% title('Pressure comparison')
% ylabel('Pressure[Bar]')
% xlabel('length')
% hold on
% it=1;
% for i=16:15:76
% for ii=1:25
%     press(i)=testcase9_pressure(i,2*ii)/1e5;
% end
% plot(linspace(1,12200,length(testcase8_pressure(1,:))),press-testcase8_pressure(i,:)./1e5,'Color',color(it,:));
% it=it+1;
% end
% 
% % 3D plots
% %massflow
% load testcase7_massflow.dat 
% %testcase1_1000_nodes_massflow=testcase1_1000_nodes_massflow(1:length(testcase2_1000_nodes_massflow(1,:)-1),:);
% surf(testcase7_massflow)
% zlabel('massflow[kg/s]')
% ylabel('time')
% xlabel('length')
% starttime=datenum('00:00:00');
% stoptime=datenum('00:10:00');
% timedata=linspace(starttime,stoptime,length(testcase7_massflow(:,1)));
% lengthdata=linspace(0,12200,length(testcase7_massflow(1,:)));
% set(gca,'YTick',1:3:30)
% set(gca, 'YTicklabel', timedata([1:3:30]));
% datetick('y',13)
% set(gca,'XTick',1:10:50)
% set(gca,'XTicklabel',lengthdata(1:10:50))
% %pressure
% load testcase7_pressure.dat
% surf(testcase7_pressure)
% zlabel('Pressure[Bar]]')
% ylabel('time')
% xlabel('length[m]')
% starttime=datenum('16:32:07');
% stoptime=datenum('17:02:07');
% timedata=linspace(starttime,stoptime,length(testcase7_pressure(:,1)));
% set(gca,'YTick',[1 46 91])
% set(gca, 'YTicklabel', timedata([1 46 91]));
% datetick('y',13)
% set(gca,'XTick',lengthdata)
% set(gca,'XTick',1:10:50)
% set(gca,'XTicklabel',lengthdata(1:10:50))